# Load utility functions from the "utils.R" file
source("utils.R")

# Define parameter values
# Parameter: Number of dimensions in synthetic data
d_values = c(100, 200)
# Parameter: Degrees of freedom in t-distribution for data generation
df_values = c(1, 2)
# Parameter: List of contamination rates for synthetic data
rho_list = c(0,0.01,0.02,0.03,0.05,0.1)
# Parameter: Number of iterations for each combination of parameters
N = 50

# Initialize an empty list to store final results
final_results <- list()

# Loop through different combinations of parameters
for (d in d_values) {
  for (df in df_values) {
    
    # Initialize an empty data frame to store results for each rate of contamination
    final_df <- t(data.frame(
      K_Slope = numeric(),
      K_Sigclust = numeric(),
      K_Silhouette = numeric(),
      ARI_Slope = numeric(),
      ARI_Sigclust = numeric(),
      ARI_Silhouette = numeric()
    ))

    # Loop through different values of the rate of contamination
    for(rho in rho_list){
      
      # Initialize a data frame to store results for each iteration
      results_df <- data.frame(
        K_Slope = numeric(),
        K_Sigclust = numeric(),
        K_Silhouette = numeric(),
        ARI_Slope = numeric(),
        ARI_Sigclust = numeric(),
        ARI_Silhouette = numeric()
      )
      cat(sprintf('Rho: %.2f\n', rho))
      
      # Loop through N iterations
      for (i in 1:N) {
        tryCatch({
          # Generate synthetic data with contamination
          X = gen_K_new(n = 100, d = d, K = 10, pcont = rho, df = df, cont = "Student", min = -10, max = 10, radius = 10)
          I = X$I
          cen = X$cen
          Tclassif = X$classif
          X = X$X
          
          cat(sprintf('Iteration: [%d/%d]', i, N))
          
          # Perform our Slope method
          a = Kmedians(X, nclust = 1:20, method = 'Offline')

          # Perform Silhouette method
          k.sil.offline = silhouette_offline(X,20)
          
          # Perform SigClust method
          sig_k = sigclust_k(X, kmax=20, alpha = 0.1)

          # Filter outliers from the true labels to evaluate ARI only on uncontaminated data
          idx = which(Tclassif != "outliers")
          
          # Store results for the current iteration
          results_df <- rbind(results_df, data.frame(K_Slope = a$Ksel, 
                                                     K_Sigclust = sig_k, 
                                                     K_Silhouette = k.sil.offline,
                                                     ARI_Slope = adjustedRandIndex(a$bestresult$cluster[idx], Tclassif[idx]),
                                                     ARI_Sigclust = adjustedRandIndex(a$allresults[[sig_k]]$cluster[idx], Tclassif[idx]),
                                                     ARI_Silhouette = adjustedRandIndex(a$allresults[[k.sil.offline]]$cluster[idx], Tclassif[idx])
                                                     
          ))
          
        }, error = function(e) {
          # Handle errors during iterations
          cat("Error in iteration ", i, ": ", conditionMessage(e), "\n")
          print(traceback())
        })
        
      }
      
      # Calculate the mean of results across all iterations
      mean_df = colMeans(results_df)
      final_df = cbind(final_df,data.frame(mean_df))
      #print(mean_df)
    }
    
    colnames(final_df) <- rho_list
    print(final_df)
    # Store the final results in the list
    final_results[[paste("d", d, "df", df, sep = "_")]] <- final_df
  }
}


################################################################################
# Analysis and summary for each simulation
################################################################################

# Print all values in final results
for (result_name in names(final_results)) {
  cat(result_name, ":\n")
  print(final_results[[result_name]])
  cat("\n")
}









